<?php 
# File: unified_antispam.php

# Plugin: ContentBan
# Ban posts or trackbacks based on whether or not they match one or more
# of a list of PCREs (Perl-Compatible Regular Expressions).
#
# This plugin is similar to the IPBan plugin, in that it uses two ban lists,
# one per-blog list and one global list.  Each lists is a plain text file
# that is formatted as one PCRE per line.  Each line must be a full PCRE 
# match expression, including delimiters and options.  So, for example,
# you might ban posts containing references to Viagra with an expression 
# like this
# | /viagra/i
#
# New expressions are added by editing the the ban files using the sidebar
# links.  Note that there is no syntax checking done on the files when they
# are edited, so you will get no warning if you enter a bad regex.  The only
# indication will be an error message displayed when someone attempts to post
# a comment or trackback.

class ContentBan extends Plugin {
	
	function ContentBan() {
		$this->plugin_desc = _("Allows you to ban comments or trackbacks that match certain regular expressions.");
		$this->plugin_version = "0.1.0";
		$this->addOption("ban_list", _("File to store list of banned regular expressions (one RE per line)."),
			"re_ban.txt", "text");
		$this->addOption("user_exempt",
		                 _("Do not apply ban to logged-in users."),
		                 false, "checkbox");
		$this->getConfig();
	}

	# Write the ban list to disk.

	function checkBan(&$obj) {
		$blog = NewBlog();
		$usr = NewUser();
		$local_list = array();
		$global_list = array();

		if ($this->user_exempt && $usr->checkLogin()) return false;
		
		if ( $blog->isBlog() && 
		     file_exists($blog->home_path.PATH_DELIM.$this->ban_list)) {
			$local_list = file($blog->home_path.PATH_DELIM.$this->ban_list);
		}
		if (file_exists(USER_DATA_PATH.PATH_DELIM.$this->ban_list)) { 
			$global_list = file(USER_DATA_PATH.PATH_DELIM.$this->ban_list);
		}
		$banned_res = array_merge($global_list, $local_list);
		$ban_post = false;
		if (strtolower(get_class($obj)) == "blogcomment") {
			$fields = array("data","subject","email","url","name");
		} else {
			$fields = array("data","url","title","blog");
		}
		foreach ($banned_res as $item) {
			foreach ($fields as $fld) {
				if (preg_match(trim($item), trim($obj->$fld))) {
					$ban_post = true;
					break;
				}
			}
		}
		return $ban_post;
	}

	# Sets the comment data to an empty string so that it cannot be added.
	function clearData(&$cmt) {
		if ($this->checkBan($cmt)) {
			$cmt->data = '';
			$cmt->subject = '';
			$cmt->email = '';
			$cmt->url = '';
			$cmt->name = '';
		}
	}

	# Sets the trackback URL to an empty string so that it cannot be added.
	function clearTBData(&$tb) {
		if ($this->checkBan($tb)) {
			$tb->url = false;
			$tb->data = '';
			$tb->title = '';
			$tb->blog = '';
		}
	}

	function sidebarLink($param) {
		global $PLUGIN_MANAGER;
		$blg = NewBlog();
		$usr = NewUser();
		$banfile = $this->ban_list;
		echo '<li><a href="'.$blg->uri('editfile', "file=$banfile").'">'.
			_("Blog RegEx blacklist").'</a></li>';
		if ($usr->isAdministrator()) {
			echo '<li><a href="'.
				make_uri(INSTALL_ROOT_URL.'pages/editfile.php',
				         array('file'=>'userdata/'.$banfile)).'">'.
				_("Global RegEx blacklist").'</a></li>';
		}
	}

}

$ban =& new ContentBan();
$ban->registerEventHandler("blogcomment", "OnInsert", "clearData");
$ban->registerEventHandler("trackback", "POSTRetreived", "clearTBData");
$ban->registerEventHandler("loginops", "PluginOutput", "sidebarLink");
?>
<?php

# Plugin: TrackbackValidator
# Validates TrackBacks when they are posted.
#
# This plugin implements a simple method for preventing TrackBack spam.  It 
# simply fetches the URL passed in the TrackBack ping and scans it for a link 
# to the entry the TrackBack was posted to.  If no such link is found, then the
# conclusion is that this TrackBack is spam and it is rejected.
#
# Note that there are two options for this plugin.  The first is an option to 
# exclude links to the same host as your blog from validation.  The second is 
# to check the ping URL for links to the entry base directory, in addition to
# the entry permalink.  Effectively, this means that URLs that link the the 
# entry permalink, the comments page, or a file uploaded to the entry will all
# pass validation.  Without this, only links to the "official" permalink will 
# pass.
#
# The idea for this plugin comes from the paper 
# "Taking TrackBack Back (from Spam)" by Gerecht et al. from Rice University.
# The paper is available at 
# <http://seclab.cs.rice.edu/proj/trackback/papers/taking-trackback-back.pdf>

class TrackbackValidator extends Plugin {

	function TrackbackValidator() {
		$this->plugin_desc = _('Allow only TrackBacks that link to your URL in the page body.');
		$this->plugin_version = '0.1.0';
		
		$this->addOption('allow_self', 
		                 _('Allow TrackBacks from your own blog'),
		                 true, 'checkbox');
		$this->addOption('base_uri',
		                 _('Allow pings from pages that link to anything under the entry, not just the permalink.'),
		                 true, 'checkbox');
		$this->getConfig();
	}
	
	function check_for_link(&$param) {
		$url = parse_url($param->url);
		if ( $this->allow_self && $url['host'] == SERVER("SERVER_NAME") ) {
			return true;
		}
		$ent = $param->getParent();
		$data = $this->fetch_page($param->url);
		
		# If the permalink is in the page, it's legitimate, so return true.
		if (strpos($data, $ent->permalink()) > 0) {
			return true;
		}
		
		# If we're also checking the base URI and that's in the page, then
		# we can also return true.
		# Note that this should return true even if the link is actually to the
		# comments page or a file under the entry rather than the permalink
		# wrapper script.  This may or may not be a good thing, hence the 
		# option.
		if ($this->base_uri && strpos($data, $ent->uri('base')) > 0) {
			return true;
		} else {
			$param->url = '';
			return false;
		}
	}
	
	function fetch_page($url) {
		if (extension_loaded('curl')) {
			
			$hnd = curl_init();
			curl_setopt($hnd, CURLOPT_URL, $url);
			curl_setopt($hnd, CURLOPT_FOLLOWLOCATION, 1);
			curl_setopt($hnd, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($hnd, CURLOPT_HEADER, 1);
			$response = curl_exec($hnd);
			
		} else {
			
			$url_bits = parse_url($url);
			$host = $url_bits['host'];
			$path = $url_bits['path'];
			$port = isset($url_bits['port']) ? $url_bits['port'] : 80;
			$query = isset($url_bits['query']) ? $url_bits['query'] : '';
			
			# Open a socket.
			$fp = @fsockopen($host, $port);
			if (!$fp) return false;
	
			# Create the HTTP request to be sent to the remote host.
			if ($query) $path .= '?'.$query;
			$data = "GET ".$path."\r\n".
					"Host: ".$host."\r\n".
					"Connection: close\r\n\r\n";
			
			# Send the data and then get back any response.
			fwrite($fp, $data);
			$response = '';

			while (! feof($fp)) {
				$s = fgets($fp);
				$response .= $s;
			}
			fclose($fp);
		}
		return $response;
	}
	
}

$plug =& new TrackbackValidator();
$plug->registerEventHandler("trackback", "POSTRetreived", "check_for_link");
?>
<?php 

# Plugin: IPBan
# Bans certain IP addresses from posting comments or trackbacks.
#
# This plugin works by merging two lists of IP addresses.  Both of these 
# files have the same name (which is configurable as an option for the 
# plugin), but one is located in the root of the blog and the other is in
# the LnBlog userdata folder.  The file format is one IP address per line.
# When the plugin runs, it reads both of these files (if they exist) and
# merges their contents.  It then rejects attempts to post comments or 
# trackbacks from any of the IP addresses in this list.
#
# Note that the IP addresses are actually interpreted as Perl-Compatible
# Regular Expressions.  This means that you can use regular expression syntax
# to ban ranges of IP addresses with a single entry.  One example might be to
# ban a subnet with an entry like this
# | 12.34.56.*
#
# There are two ways to add IP addresses to the ban list.  One is through 
# links in the footers of comments and trackbacks.  Clicking these links will
# add the exact IP address of the comment/trackback to the ban list.
# The other method is by directly editing the files using the sidebar links.
# This is the method required if you want to ban ranges of IP addresses.

class IPBan extends Plugin {
	
	function IPBan() {
		$this->plugin_desc = _("Allows you to ban IP addresses from adding comments or trackbacks.");
		$this->plugin_version = "0.2.4";
		$this->addOption("ban_list", _("File to store list of banned IPs."),
			"ip_ban.txt", "text");
		$this->addOption("admin_local", _("Show per-blog ban link when administrator"), false, "checkbox");
		$this->addOption("ban_del", _("Ban link both bans IP and deletes"), true, "checkbox");
		$this->getConfig();
	}

	# Write the ban list to disk.

	function updateList($add_list, $do_global=false) {
		global $SYSTEM;
		$blog = NewBlog();
		$usr = NewUser();
		if ( $blog->isBlog() && $SYSTEM->canModify($blog, $usr) && !$do_global) {
			$file = $blog->home_path.PATH_DELIM.$this->ban_list;
		} elseif ( $usr->checkLogin() && $usr->isAdministrator() ) {
			$file = USER_DATA_PATH.PATH_DELIM.$this->ban_list;
		} else {
			return false;
		}
		if (file_exists($file)) $list = file($file);
		else $list = array();
		$list = array_merge($list, $add_list);
		$list = array_unique($list);
		sort($list);
		$content = '';
		foreach ($list as $ip) {
			if ($content != '') $content .= "\n";
			$content .= trim($ip);
		}
		$ret = write_file($file, $content);
	}

	function addBanLink(&$cmt) {
		global $SYSTEM;
		$blog = NewBlog();
		$usr = NewUser();
		if ($SYSTEM->canModify($blog, $usr) && $usr->checkLogin()) {
			$cb_link = 
				spf_("IP: %s", $cmt->ip);
			if ($this->ban_del) {
				$cb_link_loc =
					' (<a href="'.make_uri($cmt->uri("delete"), array('banip'=>$cmt->ip)).'" '.
					'onclick="return window.confirm(\''.
					spf_("Delete %s and ban IP address %s from submitting comments or trackbacks to this blog?", $cmt->getAnchor(), $cmt->ip).
					'\');">'._("Delete &amp; Ban IP").'</a>) ';
				$cb_link_glob = ' (<a href="'.
					make_uri($cmt->uri("delete"),array('banip'=>$cmt->ip, 'global'=>'yes')).'" '.
					'onclick="return window.confirm(\''.
					spf_("Delete %s and ban IP address %s from submitting comments or trackbacks to this entire site?", $cmt->getAnchor(), $cmt->ip).
					'\');">'._("Delete &amp; Ban Globally").'</a>)';
			} else {
				$cb_link_loc =
					' (<a href="'.make_uri(false, array('banip'=>$cmt->ip)).'" '.
					'onclick="return window.confirm(\''.
					spf_("Ban IP address %s from submitting comments or trackbacks to this blog?", $cmt->ip).
					'\');">'._("Ban IP").'</a>) ';
				$cb_link_glob = ' (<a href="'.
					make_uri(false,array('banip'=>$cmt->ip, 'global'=>'yes')).
					'onclick="return window.confirm(\''.
					spf_("Ban IP address %s from submitting comments or trackbacks to this entire site?", 	$cmt->ip).
					'\');">'._("Global Ban").'</a>)';
			}

			if ($this->checkBan($cmt->ip) || $cmt->ip == GET('banip')) {
				$cb_link .= _(" (Banned)");
			} elseif ($usr->checkLogin() && $usr->isAdministrator()) {
				if ($this->admin_local) {
					$cb_link .= $cb_link_loc.$cb_link_glob;
				} else {
					$cb_link .= $cb_link_glob;
				}
			} else {
				$cb_link .= $cb_link_loc;
			}
			
			$cmt->control_bar[] = $cb_link;
		}
	}

	# Ban an IP based on a query string.

	function banIP(&$param) {
		global $SYSTEM;
		if (isset($_GET["banip"])) {
			$blog = NewBlog();
			$usr = NewUser();
			if ($SYSTEM->canModify($blog, $usr) && $usr->checkLogin()) {
				$ip = trim($_GET["banip"]);
				$global = isset($_GET["global"]);
				$this->updateList(array($ip), $global);
				#$param->redirect(make_uri());
			}
		}
	}

	function checkBan($check_ip) {
		$blog = NewBlog();
		$local_list = array();
		$global_list = array();
		if ( $blog->isBlog() && 
		     file_exists($blog->home_path.PATH_DELIM.$this->ban_list)) {
			$local_list = file($blog->home_path.PATH_DELIM.$this->ban_list);
		}
		if (file_exists(USER_DATA_PATH.PATH_DELIM.$this->ban_list)) { 
			$global_list = file(USER_DATA_PATH.PATH_DELIM.$this->ban_list);
		}
		$banned_ips = array_merge($global_list, $local_list);
		$ban_post = false;
		foreach ($banned_ips as $item) {
			if (preg_match("/^".trim($item)."$/", trim($check_ip))) {
				$ban_post = true;
				break;
			}
		}
		return $ban_post;
	}

	# Sets the comment data to an empty string so that it cannot be added.
	function clearData(&$cmt) {
		if ($this->checkBan(trim($cmt->ip))) {
			$cmt->data = '';
			$cmt->subject = '';
			$cmt->email = '';
			$cmt->homepage = '';
			$cmt->name = '';
		}
	}

	# Sets the trackback URL to an empty string so that it cannot be added.
	function clearTBData(&$tb) {
		if ($this->checkBan(trim($tb->ip))) {
			$tb->url = false;
			$tb->data = '';
			$tb->title = '';
			$tb->blog = '';
		}
	}

	function sidebarLink($param) {
		global $PLUGIN_MANAGER;
		$blg = NewBlog();
		$usr = NewUser();
		$banfile = $PLUGIN_MANAGER->plugin_config->value("ipban", 
		                                 "ban_list", "ip_ban.txt");
		echo '<li><a href="'.$blg->uri('editfile', "file=$banfile").'">'.
			_("Blog IP blacklist").'</a></li>';
		if ($usr->isAdministrator()) {
			echo '<li><a href="'.
				make_uri(INSTALL_ROOT_URL.'pages/editfile.php',
				         array('file'=>'userdata/'.$banfile)).
				'">'._("Global IP blacklist").'</a></li>';
		}
	}

}

$ban =& new IPBan();
$ban->registerEventHandler("blogcomment", "OnOutput", "addBanLink");
$ban->registerEventHandler("blogcomment", "OnInsert", "clearData");
$ban->registerEventHandler("trackback", "OnOutput", "addBanLink");
$ban->registerEventHandler("trackback", "POSTRetreived", "clearTBData");
$ban->registerEventHandler("loginops", "PluginOutput", "sidebarLink");
$ban->registerEventHandler("page", "OnOutput", "banIP");
?>
<?php

# Plugin: DisableComments
# Close comments or trackbacks on selected entries.
#
# This plugin allows you to disable comments or trackbacks on all the entries
# in a blog.  You have several options for this.  For comments, you can
# disable all comments, allow comments only by logged-in users, or use the
# per-entry setting (which is the default).  For trackbacks, you can just
# turn them all off.
#
# There is also a time-delay option.  You can just enter a number of days 
# and both comments and trackbacks will be disabled on all entries more than
# that number of days old.  
#
# Note that this plugin does dynamic disabling, so the actual entry files 
# will not be modified.  Thus, if you edit an old entry, or you will still
# the the checkbox to allow comments as enabled.  This plugin automatically
# overrides all such settings.

class DisableComments extends Plugin {

	function DisableComments() {
		$this->plugin_desc = _("Allows you to globally disable comments, trackbacks, or pingbacks for an entire blog.");
		$this->plugin_version = "0.2.0";
		$this->no_comment = "default";
		$this->no_trackback = false;
		$this->close_old = "";
		$this->addOption("no_comment", _("Allow comments"),
		                 "default", "radio", 
		                 array("default" =>_("Use per-entry setting"),
		                       "loggedin"=>_("Logged in users only"),
		                       "disable" =>_("Disable all comments")));
		$this->addOption("no_trackback", _("Disable trackbacks for all entries"),
		                 false, "checkbox");
		$this->addOption("no_pingback", _("Disable pingbacks for all entries"),
		                 false, "checkbox");
		$this->addOption("close_old", 
				_("Close all replies on entries older than this many days"),
				"", "text");
		$this->getConfig();
	}

	function disable(&$param) {
		
		if ($this->no_comment == "disable") {
			$param->allow_comment = false;
		} elseif ($this->no_comment == "loggedin") {
			$usr = NewUser();
			if (! $usr->checkLogin()) $param->allow_comment = false;
		}
		if ($this->no_trackback) $param->allow_tb = false;
		if ($this->no_pingback) $param->allow_pingback = false;

		if (is_numeric($this->close_old) && $this->close_old > 0) {
			# Subtract the number of days * 86400 seconds/day from the 
			# current time to get the target timestamp.
			$close_time = time() - $this->close_old * 86400;
			if ($param->post_ts < $close_time) {
				$param->allow_comment = false;
				$param->allow_tb = false;
				$param->allow_pingback = false;
			}
		}
	}

}
$obj =& new DisableComments();
$obj->registerEventHandler("blogentry", "InitComplete", "disable");
$obj->registerEventHandler("article", "InitComplete", "disable");

?>
